﻿Imports System.IO
Imports System.IO.Compression

''' <summary>
''' Der XMLSerializer serialisiert eine beliebige Klasse welche auch Primitiven Datentypen oder solchen welche als Serialisiert gekennzeichnet sind
''' Klasse ist generisch aufgebaut und gibt die selbe Type von Klasse zurück welche ihm übergeben wird.
'''
''' Um 2 fache Compression/Decompression erweitert, erst gzip, dann Deflatestream, um die größe des Savefile klein zu halten. Notiz:  Xml ist schneller als Json.
''' </summary>
Public Class XmlSerializer
    ''' <typeparam name="T">Den Klassentyp angeben</typeparam>
    ''' <param name="path">Der Dateipfad inkl. Dateiendung angeben</param>
    ''' <param name="instance">Die Instanz der Klasse welche serialisiert werden soll</param>
    Public Sub Serialize(Of T)(path As String, instance As T, Comp As Boolean)
        Try
            Dim dirName As String = IO.Path.GetDirectoryName(path)
            If Not Directory.Exists(dirName) Then Directory.CreateDirectory(dirName)
            File.Delete(path)
            SaveToStream(path, instance, Comp)
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Sub

    ''' <typeparam name="T">Der Typ der Klasse welche erwartet wird.</typeparam>
    ''' <param name="path">Der Pfad zur XML-Datei inkl. Dateiendung</param>
    ''' <param name="defaultInstance">Die Instanz der Klasse falls die Datei noch nicht Existiert oder nicht gefunden werden kann</param>
    ''' <returns>Gibt die Deserialissierte Klasse zurück</returns>
    Public Function DeSerialize(Of T)(path As String, defaultInstance As T, Comp As Boolean) As T
        Try
            If Not File.Exists(path) Then Return defaultInstance
            Return LoadFromStream(Of T)(path, Comp)
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Function

    ''' <typeparam name="T">Den Klassentyp angeben</typeparam>
    ''' <param name="spath">Der Stream welcher die Daten enthält (Filestream, MemoryStream,...)</param>
    ''' <param name="o">Die Instanz der Klasse welche serialisiert werden soll</param>
    Public Sub SaveToStream(Of T)(spath As String, o As T, Compress As Boolean)
        Try
            If Compress Then
                Using fs As New FileStream(spath, FileMode.OpenOrCreate)
                    Using cfs As New GZipStream(fs, CompressionMode.Compress)
                        Using dcfs As New DeflateStream(cfs, CompressionMode.Compress)
                            Dim x As New Xml.Serialization.XmlSerializer(GetType(T))
                            x.Serialize(dcfs, o)
                        End Using
                    End Using
                End Using
            Else
                Using fs As New FileStream(spath, FileMode.OpenOrCreate)
                    Dim x As New Xml.Serialization.XmlSerializer(GetType(T))
                    x.Serialize(fs, o)
                End Using
            End If
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Sub


    ''' <typeparam name="T">Der Typ der Klasse welche erwartet wird</typeparam>
    ''' <param name="spath">Der Stream welcher die Daten enthält (Filestream, MemoryStream,...)</param>
    ''' <returns>Gibt die Deserialisierte Klasse zurück</returns>
    Public Function LoadFromStream(Of T)(spath As String, Decompress As Boolean) As T
        Try
            If Decompress Then
                Using fs As New FileStream(spath, FileMode.OpenOrCreate)
                    Using cfs As New GZipStream(fs, CompressionMode.Decompress)
                        Using dcfs As New DeflateStream(cfs, CompressionMode.Decompress)
                            Dim x As New Xml.Serialization.XmlSerializer(GetType(T))
                            Return CType(x.Deserialize(dcfs), T)
                        End Using
                    End Using
                End Using
            Else
                Using fs As New FileStream(spath, FileMode.OpenOrCreate)
                    Dim x As New Xml.Serialization.XmlSerializer(GetType(T))
                    Return CType(x.Deserialize(fs), T)
                End Using
            End If
        Catch ex As Exception
            Throw New Exception(ex.Message, ex.InnerException)
        End Try
    End Function

End Class
